package FaiTS;

=head1 NAME

FaiTS - FAI Template System Helper Module

=head1 SYNOPSIS

 use FaiTS qw(dirIsWorldWritable findTemplates writeTargetFiles);
 print 'Uuuuh!' if dirIsWorldWritable("/etc");
 
 find(\&findTemplates);

 writeTargetFiles(\%targetFiles, \@classes, $prefixTemplateDir, $prefixTargetDir);

=head1 ABSTRACT

 Mostly undocumented helper functions

=head1 SEE ALSO

 http://www.yaml.org
 http://www.template-toolkit.org
 http://www.informatik.uni-koeln.de/fai/

=head1 AUTHOR

 Copyright 2006 by Ingo Wichmann <iw@linuxhotel.de>

 This software ist free software; you can redistribute it and/or modify
 it unter the same terms as Perl itself.

=cut

use 5.6.0;
use strict;
use warnings;

use base qw(Exporter);
our @EXPORT_OK = qw(writeTargetFiles findTemplates dirIsWorldWritable);

our $VERSION = '0.01';

use File::Spec::Functions qw(:ALL);
use File::Basename;
use File::Path;
use File::Find;
use File::Compare;
use File::Copy;
use File::stat;
use Template;

sub writeTargetFiles
{
  my ($targetFiles, $classes, $config, $prefixTemplateDir, $prefixTargetDir) = @_;
  TARGETFILE:
  while ( my ( $targetFile, $class) = each %$targetFiles )
  {
    # select the right template file
    my $templateFile;
    CLASS:
    foreach (reverse @$classes)
    { # the first class found wins
      if ( exists $class->{$_} )
      {
        $templateFile = "$_.tt";
        last CLASS;
      }
    }

    my $targetDir = dirname($targetFile);

    #before writing to this directory, make sure noone else can
    #see http://archive.cert.uni-stuttgart.de/archive/bugtraq/1998/12/msg00118.html
    if ( my $dir = dirIsWorldWritable( "$prefixTargetDir/$targetDir" ) )
    {
      print STDERR "Warning: $dir is group or world writable", $/;
      #print STDERR "$dir is group or world writable, refusing to create $prefixTargetDir/$targetFile", $/;
      #next TARGETFILE;
    } 

    # create target directory, if missing
    if ( not -d "$prefixTargetDir/$targetDir" )
    {
      eval { mkpath "$prefixTargetDir/$targetDir", 0, 0755 };
      if ($@)
      {
        print STDERR "Couldn't create $prefixTargetDir/$targetDir: $@";
        next TARGETFILE;
      }
    }
    # process template
    my $newTargetFile = File::Temp->new(DIR => "$prefixTargetDir/$targetDir");
    my $tt = Template->new(INCLUDE_PATH => "$prefixTemplateDir/$targetFile");
    unless (
      $tt->process( $templateFile, $config, $newTargetFile->filename )
    ) {
      warn $tt->error();
      next TARGETFILE;
    }
    # keep backup of existing files if changed
    if ( -f "$prefixTargetDir/$targetFile" and compare("$prefixTargetDir/$targetFile", $newTargetFile) )
    {
      copy("$prefixTargetDir/$targetFile","$prefixTargetDir/$targetFile.pre_tt")
        or warn "Copy of $prefixTargetDir/$targetFile to $prefixTargetDir/$targetFile.pre_tt failed: $!";
    }
    move($newTargetFile->filename, "$prefixTargetDir/$targetFile")
      or warn "move of $newTargetFile to $prefixTargetDir/$targetFile failed: $!";
  }
  return 1;
}

sub dirIsWorldWritable 
{
  my ($dir) = @_;
  #make sure noone else can write to this directory
  #see http://archive.cert.uni-stuttgart.de/archive/bugtraq/1998/12/msg00118.html
  my $parentDir;
  foreach (splitdir( rel2abs($dir) ))
  { # check all parent dirs not group/world writable
    $parentDir = catdir( $parentDir, $_ );
    #this is too paranoid, needs more work to relax it securely...
    return $parentDir 
      if 
      ( 
        -e $parentDir and 
        ( 
          not -d $parentDir 
          and not ( -o $parentDir or stat($parentDir)->uid == 0 )
          or stat($parentDir)->mode & 022 
        )
      )
  }
  return ;
}

sub findTemplates
{
  my ( $ignoreDirs, $classes, $prefixTemplateDir ) = @_;
  #Turn array into hash:
  $classes = { map { $_, 1 } @$classes };
  my $targetFiles = {};
  find( 
    sub {
      # Skip other Files and Directories
      $File::Find::prune = 1
        if $ignoreDirs->{$_};
      m!([^/]+)\.tt$! or return;
      exists $classes->{$1} or return;
      
      # remove $prefixTemplateDir in $File::Find::dir
      my @targetFile = splitdir( $File::Find::dir );
      my @prefixTemplateDir = splitdir( $prefixTemplateDir );
      my $targetFile = catdir( @targetFile[ @prefixTemplateDir .. $#targetFile ]);
      
      # add class to $targetFile entry
      if ( defined $targetFiles->{$targetFile} )
      { $targetFiles->{$targetFile}->{$1} = 1; }
      else
      { $targetFiles->{$targetFile} = { $1 => 1 }; }
    },
    $prefixTemplateDir
  );
  return $targetFiles;
}

1;

